/**
 * Copyright (C) 2017 Wasabeef
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var RE = {};

RE.currentSelection = {
    "startContainer": 0,
    "startOffset": 0,
    "endContainer": 0,
    "endOffset": 0};

RE.editor = document.getElementById('editor');

document.addEventListener("selectionchange", function() { RE.backuprange(); });

document.addEventListener("paste", function(e) {
    if (e.clipboardData) {
        // 阻止默认行为
        e.preventDefault();
        var clipboardData = e.clipboardData;
        // 获取剪贴板的文本
        var text = clipboardData.getData('text/plain');
        if (window.getSelection && text !== '' && text !== null) {
            document.execCommand('insertText', false, text);
        }　　　　
    }
});
// Initializations
RE.callback = function() {
    window.location.href = "re-callback://" + encodeURI(RE.getHtml());
}
RE.setHtml = function(contents) {
    var contents = filterXSS(contents.replace(/\+/g, '%20'));
    RE.editor.innerHTML = decodeURIComponent(contents);
}

RE.getHtml = function() {
    return RE.editor.innerHTML;
}

RE.getText = function() {
    return RE.editor.innerText;
}

RE.setBaseTextColor = function(color) {
    RE.editor.style.color  = color;
}

RE.setBaseFontSize = function(size) {
    RE.editor.style.fontSize = size;
}

RE.setPadding = function(left, top, right, bottom) {
  RE.editor.style.paddingLeft = left;
  RE.editor.style.paddingTop = top;
  RE.editor.style.paddingRight = right;
  RE.editor.style.paddingBottom = bottom;
}

RE.setBackgroundColor = function(color) {
    document.body.style.backgroundColor = color;
}

RE.setBackgroundImage = function(image) {
    RE.editor.style.backgroundImage = image;
}

RE.setWidth = function(size) {
    RE.editor.style.minWidth = size;
}

RE.setHeight = function(size) {
    RE.editor.style.height = size;
}

RE.setTextAlign = function(align) {
    RE.editor.style.textAlign = align;
}

RE.setVerticalAlign = function(align) {
    RE.editor.style.verticalAlign = align;
}

RE.setPlaceholder = function(placeholder) {
    RE.editor.setAttribute("placeholder", placeholder);
}

RE.setInputEnabled = function(inputEnabled) {
    RE.editor.contentEditable = String(inputEnabled);
}

RE.undo = function() {
    document.execCommand('undo', false, null);
    RE.enabledEditingItems({});
}

RE.redo = function() {
    document.execCommand('redo', false, null);
    RE.enabledEditingItems({});
}

RE.setBold = function() {
    document.execCommand('bold', false, null);
    RE.enabledEditingItems({});
}

RE.setItalic = function() {
    document.execCommand('italic', false, null);
    RE.enabledEditingItems({});
}

RE.setSubscript = function() {
    document.execCommand('subscript', false, null);
    RE.enabledEditingItems({});
}

RE.setSuperscript = function() {
    document.execCommand('superscript', false, null);
    RE.enabledEditingItems({});
}

RE.setStrikeThrough = function() {
    document.execCommand('strikeThrough', false, null);
    RE.enabledEditingItems({});
}

RE.setUnderline = function() {
    document.execCommand('underline', false, null);
    RE.enabledEditingItems({});
}

RE.setBullets = function() {
    document.execCommand('insertUnorderedList', false, null);
    RE.enabledEditingItems({});
}

RE.setNumbers = function() {
    document.execCommand('insertOrderedList', false, null);
    RE.enabledEditingItems({});
}

RE.setTextColor = function(color) {
    RE.restorerange();
    document.execCommand("styleWithCSS", null, true);
    document.execCommand('foreColor', false, color);
    document.execCommand("styleWithCSS", null, false);
}

RE.setTextBackgroundColor = function(color) {
    RE.restorerange();
    document.execCommand("styleWithCSS", null, true);
    document.execCommand('hiliteColor', false, color);
    document.execCommand("styleWithCSS", null, false);
}

RE.setFontSize = function(fontSize){
    document.execCommand("fontSize", false, fontSize);
}

RE.setHeading = function(heading) {
    document.execCommand('formatBlock', false, '<h'+heading+'>');
    RE.enabledEditingItems({});
}

RE.setIndent = function() {
    document.execCommand('indent', false, null);
    RE.enabledEditingItems({});
}

RE.setOutdent = function() {
    document.execCommand('outdent', false, null);
    RE.enabledEditingItems({});
}

RE.setJustifyLeft = function() {
    document.execCommand('justifyLeft', false, null);
    RE.enabledEditingItems({});
}

RE.setJustifyCenter = function() {
    document.execCommand('justifyCenter', false, null);
    RE.enabledEditingItems({});
}

RE.setJustifyRight = function() {
    document.execCommand('justifyRight', false, null);
    RE.enabledEditingItems({});
}

RE.setBlockquote = function() {
    document.execCommand('formatBlock', false, '<blockquote>');
    RE.enabledEditingItems({});
}

RE.insertImage = function(url, alt) {
    var html = '<img src="' + url + '" alt="' + alt + '" />';
    RE.insertHTML(html);
}

RE.insertHTML = function(html) {
    RE.restorerange();
    document.execCommand('insertHTML', false, html);
}

RE.insertLink = function(url, title) {
    RE.restorerange();
    var sel = document.getSelection();
    if (sel.toString().length == 0) {
        document.execCommand("insertHTML", false, "<a href='" + url + "'>" + title + "</a>");
    } else if (sel.rangeCount) {
        var el = document.createElement("a");
        el.setAttribute("href", url);
        el.setAttribute("title", title);

        var range = sel.getRangeAt(0).cloneRange();
        range.surroundContents(el);
        sel.removeAllRanges();
        sel.addRange(range);
    }
    RE.onWebContentsChanged();
}

RE.setTodo = function(text) {
    var html = '<input type="checkbox" name="'+ text +'" value="'+ text +'"/> &nbsp;';
    document.execCommand('insertHTML', false, html);
}

RE.prepareInsert = function() {
    RE.backuprange();
}

RE.backuprange = function(){
    var selection = window.getSelection();
    if (selection.rangeCount > 0) {
      var range = selection.getRangeAt(0);
      RE.currentSelection = {
          "startContainer": range.startContainer,
          "startOffset": range.startOffset,
          "endContainer": range.endContainer,
          "endOffset": range.endOffset};
    }
}

RE.restorerange = function(){
    var selection = window.getSelection();
    selection.removeAllRanges();
    var range = document.createRange();
    range.setStart(RE.currentSelection.startContainer, RE.currentSelection.startOffset);
    range.setEnd(RE.currentSelection.endContainer, RE.currentSelection.endOffset);
    selection.addRange(range);
}

RE.enabledEditingItems = function(e) {
    var items = [];
    if (document.queryCommandState('bold')) {
        items.push('bold');
    }
    if (document.queryCommandState('italic')) {
        items.push('italic');
    }
    if (document.queryCommandState('subscript')) {
        items.push('subscript');
    }
    if (document.queryCommandState('superscript')) {
        items.push('superscript');
    }
    if (document.queryCommandState('strikeThrough')) {
        items.push('strikeThrough');
    }
    if (document.queryCommandState('underline')) {
        items.push('underline');
    }
    if (document.queryCommandState('insertOrderedList')) {
        items.push('orderedList');
    }
    if (document.queryCommandState('insertUnorderedList')) {
        items.push('unorderedList');
    }
    if (document.queryCommandState('justifyCenter')) {
        items.push('justifyCenter');
    }
    if (document.queryCommandState('justifyFull')) {
        items.push('justifyFull');
    }
    if (document.queryCommandState('justifyLeft')) {
        items.push('justifyLeft');
    }
    if (document.queryCommandState('justifyRight')) {
        items.push('justifyRight');
    }
    if (document.queryCommandState('insertHorizontalRule')) {
        items.push('horizontalRule');
    }
    var formatBlock = document.queryCommandValue('formatBlock');
    if (formatBlock.length > 0) {
        items.push(formatBlock);
    }

    window.location.href = "re-state://" + encodeURI(items.join(','));
}

RE.focus = function() {
    var range = document.createRange();
    range.selectNodeContents(RE.editor);
    range.collapse(false);
    var selection = window.getSelection();
    selection.removeAllRanges();
    selection.addRange(range);
    RE.editor.focus();
}

RE.blurFocus = function() {
    RE.editor.blur();
}

RE.removeFormat = function() {
    document.execCommand('removeFormat', false, null);
}

// Event Listeners
RE.editor.addEventListener("input", RE.callback);
RE.editor.addEventListener("keyup", function(e) {
    var KEY_LEFT = 37, KEY_RIGHT = 39;
    if (e.which == KEY_LEFT || e.which == KEY_RIGHT) {
        RE.enabledEditingItems(e);
    }
//    var KEY_DELETE = 8;
//    if (e.which == KEY_DELETE) {
//        RE.onWebMentionDeleteChanged(e);
//    }
});
RE.editor.addEventListener("click", RE.enabledEditingItems);

// 图片点击监听
RE.onImageClickCallback = function() {
    var $imageItems = [];
    var $img = document.getElementsByTagName('img');
    for(var p in $img) {
        if (typeof $img[p] === 'object') {
            $imageItems.push($img[p].src);
        }
    }
    var $imageUrls = $imageItems.join("|");
    for(var p in $img) {
        if (typeof $img[p] === 'object') {
            $img[p].onclick = function(e) {
                window.androidJs.onImageClick($imageUrls, e.srcElement.src);
            };
        }
    }
}

// 图片变化监听
RE.onImageChangedCallback = function() {
    var $imageItems = [];
    var $count = 0;
    var $img = document.getElementsByTagName('img');
    for (var p in $img) {
        if (typeof $img[p] === 'object') {
            if ($img[p].getAttribute("class") == 'image_src') {
                $imageItems.push($img[p].src);
                $count++;
            }
        }
    }
    var $imageUrls = $imageItems.join("|");
    window.androidJs.onImageChanged($imageUrls, $count);
}

RE.setParagraph = function() {
    document.execCommand('formatBlock', false, '<p>');
    RE.enabledEditingItems({});
}

RE.insertImage2 = function(url, alt) {
    var html='<br class="edit_newline"><div class="image_container" contenteditable="false"><p class="image_delete_container" contenteditable="false"><img class="image_delete" src="https://img0.jihai8.com/jhimg/32acf09889442330b20b345ecf79928b.png"/></p><img class="image_src" src="' + url + '" alt="' + alt + '"/></div><br class="edit_newline">';
    RE.insertHTML(html);
    RE.addOnImageEditListener();

    RE.onWebContentsChanged();
}

RE.addOnImageEditListener = function() {
    var container = document.getElementsByClassName("image_delete_container");
    for (var i in container) {
        container[i].onclick = function() {
            this.parentNode.parentNode.removeChild(this.parentNode);

            RE.onWebContentsChanged();
        };
    }
}

RE.onWebContentsChanged = function() {
    var webContents = RE.getHtml();
    var webTexts = RE.getText();
    window.androidJs.onWebContentsChanged(webContents, webTexts);
}

RE.onWebStatesChanged = function() {
    var items = [];
    if (document.queryCommandState('bold')) {
        items.push('bold');
    }
    if (document.queryCommandState('italic')) {
        items.push('italic');
    }
    if (document.queryCommandState('subscript')) {
        items.push('subscript');
    }
    if (document.queryCommandState('superscript')) {
        items.push('superscript');
    }
    if (document.queryCommandState('strikeThrough')) {
        items.push('strikeThrough');
    }
    if (document.queryCommandState('underline')) {
        items.push('underline');
    }
    if (document.queryCommandState('insertOrderedList')) {
        items.push('orderedList');
    }
    if (document.queryCommandState('insertUnorderedList')) {
        items.push('unorderedList');
    }
    if (document.queryCommandState('justifyCenter')) {
        items.push('justifyCenter');
    }
    if (document.queryCommandState('justifyFull')) {
        items.push('justifyFull');
    }
    if (document.queryCommandState('justifyLeft')) {
        items.push('justifyLeft');
    }
    if (document.queryCommandState('justifyRight')) {
        items.push('justifyRight');
    }
    if (document.queryCommandState('insertHorizontalRule')) {
        items.push('horizontalRule');
    }
    var formatBlock = document.queryCommandValue('formatBlock');
    if (formatBlock.length > 0) {
        items.push(formatBlock);
    }
    var webStates = items.join(',');
    var webTexts = RE.getText();
    window.androidJs.onWebStatesChanged(webStates, webTexts);
}

RE.removeClickListener = function() {
    RE.editor.removeEventListener("click", RE.enabledEditingItems);
}

RE.insertMention = function(userId, userName) {
    var html='&nbsp;<span class="mention" contenteditable="false" id="' + userId +'">@' + userName + '</span>&nbsp;';
    RE.insertHTML(html);

    RE.onWebContentsChanged();
}

RE.addOnMentionClickListener = function() {
    var container = document.getElementsByClassName("mention");
    for (var i in container) {
        container[i].onclick = function() {
            var userId = this.id;
            if (typeof(userId) != "undefined") {
                var matchText = this.innerText;
                window.androidJs.onWebMentionClick(userId, matchText);
            }
        };
    }
}

RE.onWebMentionDataChanged = function() {
    var container = document.getElementsByClassName("mention");
    var items = [];
    for (var i in container) {
        var item = container[i];
        var userId = item.id;
        if (typeof(userId) != "undefined") {
            var matchText = item.innerText;
            var itemValue = '' + userId + ',' + matchText;
            items.push(itemValue);
        }
    }
    window.androidJs.onWebMentionDataChanged(items);
}

RE.onWebMentionDeleteChanged = function(e) {
    // 1.由于在创建时默认会在 @xxx 后添加一个空格，所以当得知光标位于 @xxx 之后的一个第一个字符后并按下删除按钮时，应该将光标前的 @xxx 给删除
    // 2.当光标位于 @xxx 中间时，按下删除按钮时应该将整个 @xxx 给删除
    var selection = window.getSelection();
    var range = selection.getRangeAt(0);
    var removeNode = null;
    if (range.startOffset <= 1 && range.startContainer.parentElement.className != "mention") {
        removeNode = range.startContainer.previousElementSibling;
    }
    if (range.startContainer.parentElement.className == "mention") {
        removeNode = range.startContainer.parentElement;
    }
    if (removeNode) {
        RE.editor.removeChild(removeNode);
    }
}
